@@ -2,7 +2,6 @@ import glob
 import json
 import logging
 import os
-import re
 import sh
 import tempfile
 import textwrap
@@ -30,16 +29,13 @@ class Ansible(object):
         if inventories:
             self.inventories.extend([f'-i{i}' for i in inventories])
 
-    def get_env(self):
-        return {
+        self.ansible_env = {
             'SHARE_DIR': self.share_dir,
             'CONFIG_DIR': self.config_dir,
             'ANSIBLE_NOCOLOR': 'true',
-            'ANSIBLE_LOAD_CALLBACK_PLUGINS': 'yes',
-            'ANSIBLE_STDOUT_CALLBACK': 'json',
         }
 
-    def bake(self, verbose=True):
+    def bake(self, env=None, verbose=True):
         args = [
             '--extra-vars', f'enough_domain_config_directory={self.config_dir}',
         ]
@@ -47,11 +43,15 @@ class Ansible(object):
         if self.vault_password_option():
             args.append(self.vault_password_option())
         logger = logging.getLogger(__name__)
-        ansible_env = os.environ.copy()
-        ansible_env.update(self.get_env())
+
+        # Fetch environment variables as late as possible
+        _env = os.environ.copy()
+        _env.update(self.ansible_env)
+        if env:
+            _env.update(env)
         kwargs = dict(
             _truncate_exc=False,
-            _env=ansible_env,
+            _env=_env,
         )
         if verbose:
             kwargs.update(
@@ -75,9 +75,12 @@ class Ansible(object):
         ] + self.inventories
         if self.vault_password_option():
             args.append(self.vault_password_option())
+        _env = os.environ.copy()
+        _env.update(ANSIBLE_VERBOSE_TO_STDERR='yes')
         i = sh.ansible_inventory(
             *args,
             _cwd=self.share_dir,
+            _env=_env,
         )
         return json.loads(i.stdout.decode('utf-8'))
 
@@ -148,8 +151,18 @@ class Ansible(object):
         ]
         if hosts:
             args.extend(('--limit', ','.join(hosts)))
-        r = self.bake(verbose=False)(*args)
-        out = json.loads(str(r))['plays'][0]['tasks'][0]['hosts']
+        env = {
+            'ANSIBLE_LOAD_CALLBACK_PLUGINS': 'yes',
+            'ANSIBLE_STDOUT_CALLBACK': 'json',
+            'ANSIBLE_VERBOSE_TO_STDERR': 'yes',
+        }
+        r = self.bake(env=env, verbose=False)(*args)
+        return self.extract_values(r, variable)
+
+    @staticmethod
+    def extract_values(result, variable):
+        # There is one play and we want to fetch the output of the last task.
+        out = json.loads(str(result))['plays'][0]['tasks'][-1]['hosts']
         values = {k: v.get(variable) for k, v in out.items()}
         for k, v in values.items():
             if v == 'VARIABLE IS NOT DEFINED!':
@@ -164,6 +177,10 @@ class Playbook(Ansible):
     class NoPasswordException(Exception):
         pass
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.ansible_env['ANSIBLE_ROLES_PATH'] = self.roles_path(self.share_dir)
+
     @staticmethod
     def is_encrypted(p):
         if not os.path.exists(p):
@@ -208,12 +225,6 @@ class Playbook(Ansible):
         r.append(f'{d}/playbooks/wazuh/wazuh-ansible/roles/wazuh')
         return ":".join(r)
 
-    def get_env(self):
-        ansible_env = super().get_env().copy()
-        ansible_env.pop('ANSIBLE_STDOUT_CALLBACK')
-        ansible_env.update(ANSIBLE_ROLES_PATH=self.roles_path(self.share_dir))
-        return ansible_env
-
     def run_from_cli(self, **kwargs):
         if not kwargs['args']:
             args = [
@@ -241,7 +252,7 @@ class Playbook(Ansible):
         :type host: str
 
         :raises: :class:`sh.ErrorReturnCode_1`: when the host is unknown
-        :raises: :class:`sh.ErrorReturnCode_2`: when the host is known but the
+        :raises: :class:`AnsibleVariableNotFound`: when the host is known but the
             variable is undefined
 
         :returns: the value of the rendered variable.
@@ -261,38 +272,38 @@ class Playbook(Ansible):
         :type hosts: list of str
 
         :raises: :class:`sh.ErrorReturnCode_1`: when the host is unknown
-        :raises: :class:`sh.ErrorReturnCode_2`: when the host is known but the
+        :raises: :class:`AnsibleVariableNotFound`: when the host is known but the
             variable is undefined
 
         :returns: a dict where keys are inventory hostnames and values are the
             rendered variable.
         """
         with tempfile.NamedTemporaryFile() as f:
-            # the sourrounding "> <" are to prevent conversion to int, list or whatever
-            playbook = textwrap.dedent("""
+            playbook = textwrap.dedent(f"""
             ---
             - hosts: all
               gather_facts: false
               serial: 1
 
               roles:
-                - role: "{{ rolevar }}"
+                - role: "{role}"
                   when: false
 
               tasks:
                 - name: print variable
                   debug:
-                    msg: ">{{ ansible_play_batch[0] }}:{{ variable }}<"
+                    var: "{variable}"
             """)
             f.write(bytearray(playbook, 'utf-8'))
             f.flush()
             args = [
-                '-e', f'rolevar={role}',
-                '-e', 'variable={{ ' + variable + ' }}',
                 '--limit', ','.join(hosts),
                 f.name,
             ]
-            r = self.bake(verbose=False)(*args)
-            return dict(re.findall(r'.*"msg": ">(.*?):(.*)<"$',
-                                   r.stdout.decode('utf-8'),
-                                   re.MULTILINE))
+            env = {
+                'ANSIBLE_LOAD_CALLBACK_PLUGINS': 'yes',
+                'ANSIBLE_STDOUT_CALLBACK': 'json',
+                'ANSIBLE_VERBOSE_TO_STDERR': 'yes',
+            }
+            r = self.bake(env=env, verbose=False)(*args)
+            return self.extract_values(r, variable)
