@@ -1,6 +1,16 @@
+import os
+import yaml
+from enough import settings
+
+
 def set_options(parser):
+    i = f'{settings.CONFIG_DIR}/inventory/group_vars/all/infrastructure.yml'
+    driver = 'openstack'
+    if os.path.exists(i):
+        driver = yaml.safe_load(open(i).read()).get('infrastructure_driver', 'openstack')
+
     parser.add_argument('--driver',
-                        default='openstack',
+                        default=driver,
                         choices=['libvirt', 'openstack'])
     parser.add_argument('--inventory', action='append')
     o = parser.add_argument_group(title='OpenStack',
