@@ -1,4 +1,4 @@
-testinfra_hosts = ['ansible://icinga-host']
+testinfra_hosts = ['ansible://website-host']
 
 
 def test_openvpnclient_route(host):
