@@ -8,8 +8,7 @@ IcingaHelper.icinga_host = 'bind-host'
 class TestChecks(IcingaHelper):
 
     def test_host(self):
-        r = self.get_client().objects.get('Host', 'packages-host')
-        assert r['attrs']['name'] == 'packages-host'
+        assert 'packages-host' in self.get_hosts(host='packages-host')
 
     def test_service(self, host):
         assert self.is_service_ok('packages-host!Packages')
